# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,Rprofmem)
S3method(c,Rprofmem)
S3method(print,Rprofmem)
S3method(subset,Rprofmem)
S3method(total,Rprofmem)
export(profmem)
export(profmem_begin)
export(profmem_depth)
export(profmem_end)
export(profmem_resume)
export(profmem_status)
export(profmem_suspend)
export(readRprofmem)
export(total)
importFrom(utils,Rprofmem)
importFrom(utils,file_test)
